/* File BRMGRPH.C  Graphics for broom balancing  */

#ifndef BRMGRPH_C
#define BRMGRPH_C

#include <stdio.h> /* for sprintf */
#include <math.h>  /* for sin, cos */
#include "ut.h"
#include "brm.h"
#include "brmgrcn.c"   /*includes <graphics.h> */

void write_brm_graph_screen (float x,float t,float f,int itr,short *page);

/*  Implementation */

int x_to_window_h_pos (float x)
   {
   int	min_h_pos,max_h_pos,track_width,val;
   /* h_pos is center of cart */
   min_h_pos = cart_half_width;
   max_h_pos = window_width - cart_half_width;
   track_width = max_h_pos - min_h_pos;
   val = (x - x_pos_min)/(x_pos_max - x_pos_min) *
	       track_width + min_h_pos;
   if (val < min_h_pos) val = min_h_pos;
   if (val > max_h_pos) val = max_h_pos;
   return val;
   }

void draw_window_border (void)
   {
   setcolor (border_color);
   rectangle (window_left - 3,window_top - 3,window_right + 3,
	  window_bottom + 3);
   rectangle (window_left - 1,window_top - 1,window_right + 1,
	  window_bottom + 1);
   }

void draw_cart_body (int h_pos)
   {
   int  cart_left,cart_right;
   cart_left = h_pos - cart_half_width;
   cart_right = h_pos + cart_half_width;
   setcolor (cart_color);
   rectangle (cart_left,cart_top,cart_right,cart_bottom);
   }

void draw_wheels (int h_pos)
   {
   int left_axle_pos,right_axle_pos;
   setcolor (wheel_color);
   left_axle_pos = h_pos - half_wheelbase;
   right_axle_pos = h_pos + half_wheelbase;
   circle (left_axle_pos,window_ht - axle_ht,wheel_radius);
   circle (right_axle_pos,window_ht - axle_ht,wheel_radius);
   }

void draw_pivot (int h_pos)
   {
   const pivot_pt_radius = 1;
   int pivot_left,pivot_right;
   setcolor (pivot_color);
   pivot_left = h_pos - broom_half_width;
   pivot_right = h_pos + broom_half_width;
   rectangle (pivot_left,pivot_top,pivot_right,pivot_bottom);
   circle (h_pos,pivot_top,broom_half_width);
   setcolor (pivot_pt_color);
   circle (h_pos,pivot_top,pivot_pt_radius);
   putpixel (h_pos,pivot_top,pivot_pt_color);
   }

const float broom_half_angle = 0.04;  /* Radians */

void broom_xy_to_screen (float brx,float bry,int h_pos,int *brh,int *brv)
   {
   float x_fract,y_fract;
   int min_h_scrn_pos,h_scrn_width;
   x_fract = (brx + 1.0)/2.0;
   h_scrn_width = 2 * broom_length_h;
   min_h_scrn_pos = h_pos - broom_length_h;
   *brh = ceil(x_fract * h_scrn_width) + min_h_scrn_pos;
   y_fract = (bry + broom_half_angle)/(1.0 + broom_half_angle);
   *brv = (pivot_top + broom_half_width) - ceil(y_fract * broom_length_v);
   return;
   }

void draw_broom (int h_pos,float angle)
   {
   float broom_x,broom_y;
   int broom_stick [10];  /* polygon points list */

   broom_x = sin (angle - broom_half_angle);
   broom_y = cos (angle - broom_half_angle);
   broom_xy_to_screen (broom_x,broom_y,h_pos,
	     &broom_stick [0],&broom_stick[1]);
   broom_x = sin (angle + broom_half_angle);
   broom_y = cos (angle + broom_half_angle);
   broom_xy_to_screen (broom_x,broom_y,h_pos,
	     &broom_stick [2],&broom_stick[3]);
   broom_x = cos (angle) * broom_half_angle;  /* small angle; sin (angle)=angle */
   broom_y = -sin (angle) * broom_half_angle;
   broom_xy_to_screen (broom_x,broom_y,h_pos,
	      &broom_stick [4],&broom_stick[5]);
   broom_x = -broom_x;
   broom_y = -broom_y;
   broom_xy_to_screen (broom_x,broom_y,h_pos,
	      &broom_stick [6],&broom_stick[7]);
   broom_stick [8] = broom_stick [0];
   broom_stick [9] = broom_stick [1];
   setcolor (broom_color);
   drawpoly (5,broom_stick);
   return;
   }

#define CLIP_ON 1

void draw_cart_window (float x,float theta)
   {
   int cart_h_pos;
   draw_window_border();
   setviewport (window_left,window_top,window_right,window_bottom,CLIP_ON);
   cart_h_pos = x_to_window_h_pos (x);
   draw_cart_body (cart_h_pos);
   draw_wheels (cart_h_pos);
   draw_broom (cart_h_pos,theta);
   draw_pivot (cart_h_pos);
   setviewport (0,0,getmaxx(),getmaxy(),CLIP_ON);
   }

void draw_force_thrm (float x,float f_mag)
   {
   int i,f_thrm_pt,char_width;
   float f_fract;
   char num_str[6];
   char back_block [3];
   char fore_block [3];
   back_block[0] = BLOCK_1;
   back_block[1] = BLOCK_1;
   back_block[2] = '\0';
   fore_block[0] = BLOCK_4;
   fore_block[1] = BLOCK_4;
   fore_block[2] = '\0';
   setcolor (f_thrm_back_color);
   settextstyle (DEFAULT_FONT,VERT_DIR,1);
   char_width = textwidth ("A");
   moveto (f_thrm_left + char_width,f_thrm_top);
   for (i = 1;i <= 20; i++) {
       outtext (back_block);
       moverel (char_width,0);
       }
   rectangle (f_thrm_left,f_thrm_top,f_thrm_right,f_thrm_bottom);
   setcolor (f_thrm_fore_color);
   moveto (f_zero_pos,f_thrm_top - 2);
   lineto (f_zero_pos,f_thrm_bottom + 2);
   f_fract = x / f_mag;  /* Between -1 and 1 */
   if (f_fract < -1.0) f_fract = -1.0;
   if (f_fract > 1.0) f_fract = 1.0;
   f_thrm_pt =  fabs (f_fract) * 10;
   if (f_thrm_pt > 10) f_thrm_pt = 10;
   if (f_fract < 0.0) {
       moveto (f_zero_pos,f_thrm_top);
       for (i = 1;i <= f_thrm_pt; i++) {
	   outtext (fore_block);
	   moverel (-char_width,0);
	   }
       }
   else {     /* f_fract >= 0.0 */
	moveto (f_zero_pos + char_width,f_thrm_top);
	for (i = 1;i <= f_thrm_pt; i++) {
	     outtext (fore_block);
	     moverel (char_width,0);
	     }
	}
    setcolor (text_color);
    settextstyle (DEFAULT_FONT,HORIZ_DIR,1);
    settextjustify (CENTER_TEXT,TOP_TEXT);
    outtextxy (f_zero_pos,f_thrm_bottom + 15,"FORCE");
    sprintf(num_str,"%3.1f",-f_mag);
    outtextxy (f_thrm_left,f_thrm_bottom + 5,num_str);
    sprintf(num_str,"%3.1f",f_mag);
    outtextxy (f_thrm_right,f_thrm_bottom + 5,num_str);
    outtextxy (f_zero_pos,f_thrm_bottom + 5,"0");
    }

void write_brm_graph_screen (float x,float t,float f,int itr,short *page)
   {
   short active_page,visual_page;
   char disp_str[18];
   *page = *page + 1;
   visual_page = *page % 2;  /* % = mod operator */
   active_page = (*page + 1) % 2;
   setvisualpage (visual_page);
   setactivepage (active_page);
   cleardevice();
   settextstyle (DEFAULT_FONT,HORIZ_DIR,1);
   settextjustify (LEFT_TEXT,TOP_TEXT);
   draw_cart_window (x,t);
   draw_force_thrm (f,force_mag);
   settextjustify (LEFT_TEXT,TOP_TEXT);
   setcolor (text_color);
   sprintf (disp_str,"ITERS: %5d",itr);
   outtextxy (20,window_bottom + 20,disp_str);
   setvisualpage (active_page);
   }

#endif
