/*  File DYNGRAPH.C  Graph Dynamical System  */

#ifndef DYNGRAPH_C
#define DYNGRAPH_C

#include <conio.h>
#include <math.h>

#include "uttypes.h"
#include "utgr.h"
#include "dyn.h"

void graph_dynamical_system (graph_setup_rec *the_graph_setup,
     dyn_sys_proc dyn_sys);

/* Implementation */

void graph_dynamical_system (graph_setup_rec *the_graph_setup,
     dyn_sys_proc dyn_sys) {
/* Define number of colors used for display of trajectory */
#define NO_OF_COLORS 12
     const unsigned short color_array [NO_OF_COLORS] = {
       YELLOW,GREEN,CYAN,RED,MAGENTA,LIGHTGRAY,LIGHTGREEN,
       LIGHTBLUE,LIGHTCYAN,LIGHTRED,LIGHTMAGENTA,WHITE};
     const int prelim_iters = 100;
     const float too_big = 1e+6;
     float x_old = 0.0,y_old = 0.0,x_new = 0.0,y_new = 0.0;
     float x_min = 0.0,x_max = 0.0,y_min = 0.0,y_max = 0.0;
     long int iter = 0;
     unsigned short color = 0;
  dyn_sys (x_old,y_old,&x_new,&y_new);
  x_min = x_new;
  x_max = x_new;
  y_min = y_new;
  y_max = y_new;
  clrscr ();
  gotoxy (1,5);
  cprintf ("Determining max-min values...");
  for (iter = 1;iter <= prelim_iters;iter++) {
      dyn_sys (x_old,y_old,&x_new,&y_new);
      if ((fabs (x_new) > too_big) || (fabs(y_new) > too_big)) {
	 gotoxy (1,7);
	 cprintf ("This system diverges.  Press key to cancel: ");
	 getch();
	 return;
	 }  /* if */
      if (x_new < x_min) x_min = x_new;
      if (x_new > x_max) x_max = x_new;
      if (y_new < y_min) y_min = y_new;
      if (y_new > y_max) y_max = y_new;
      x_old = x_new;
      y_old = y_new;
      }  /* iter */
  if (the_graph_setup->scale_window_from_data) {
     the_graph_setup->x_min = x_min;
     the_graph_setup->x_max = x_max;
     the_graph_setup->y_min = y_min;
     the_graph_setup->y_max = y_max;
     }
  if (!graph_setup (*the_graph_setup)) return;
  iter = 0;
  draw_border(*the_graph_setup);
  if (the_graph_setup->label_axes)
     graph_x_y_labels (*the_graph_setup);
  do {
     iter++;
     dyn_sys (x_old,y_old,&x_new,&y_new);
     color = (iter/1000) % NO_OF_COLORS;
     if (color_array[color] == the_graph_setup->back_color)
	color = (color + 1) % NO_OF_COLORS;
     xy_to_screen (x_new,y_new,color_array [color]);
     graph_iter (*the_graph_setup,iter);
     x_old = x_new;
     y_old = y_new;
  } while ((!kbhit()) && (fabs(x_new) < too_big) && (fabs(y_new) < too_big));
  getch ();
  outtextxy (gr_iter_col,gr_done_row,"Done");
  getch();
  closegraph();
  textmode (LASTMODE);
  return;
  }  /* end proc */

#endif