/*  File FFTRAIN_C  Train a feedforward network  */

#ifndef FFTRAIN_C
#define FFTRAIN_C

#include <conio.h>
#include <alloc.h>

#include "ut.h"
#include "trn.h"
#include "ff.h"

void train_network_from_data (ff_setup_rec setup,
   training_proc the_training_proc);
void standard_data_input (ff_setup_rec setup,training_rec tr_rec,
   int item,f_vector x_in,f_vector desired_out);

/*  Implementation  */

void train_network_from_data (ff_setup_rec setup,
   training_proc the_training_proc) {
   weights_rec weights = {0,NULL,NULL};
   training_rec tr_rec;
   int no_of_training_items;
   clrscr();
   textcolor (YELLOW);
   gotoxy (1,1);
   cprintf ("Feedforward Neural Network Training Procedure\r\n\n");
   if (setup.files.out_wts_file_name[0] == '\0') {
      cputs ("\r\nOutput weights file name is blank.\r\n"
	     "Press key to cancel: ");
      getch();
      return;
      }
   if (setup.files.training_file_name[0] == '\0') {
      cputs ("\r\nTraining file name is blank.\r\n"
	     "Press key to cancel: ");
      getch();
      return;
      }
   if (!setup.in_weights_from_file) {
      cputs ("\r\nInput weights file will not be used.\n");
      cprintf ("\r\nFile %s will be created/overwritten with new weights.",
	 setup.files.out_wts_file_name);
      cputs ("\r\nIf this is not OK, press ESC; press any other key to continue: ");
      if (getch() == ESC_KEY) return;
      }
  if (!process_training_file (setup.files.training_file_name,
      setup.no_of_inputs,setup.no_of_outputs,&tr_rec))
      goto exit_proc;
  normalize_training_set (setup.no_of_inputs,setup.no_of_outputs,
      setup.norm_in_range,setup.norm_in_min,setup.norm_out_range,
      setup.norm_out_min,tr_rec);

  weights = initialize_weights (setup,RANDOM); /* Allocates and initializes */
  clrscr();
  gotoxy (1,5);
  if (setup.in_weights_from_file)
     if (!read_weights (&weights,setup)) goto exit_proc;
  cputs ("\r\n");
  the_training_proc (setup,weights,tr_rec,standard_data_input);
exit_proc:
   free_training_pointers (&tr_rec);
   free_weights (&weights);
   textmode (LASTMODE);
   return;
   }  /* end proc */

void standard_data_input (ff_setup_rec setup,training_rec tr_rec,int item,
	    f_vector x_in,f_vector desired_out) {
     copy_f_vector (tr_rec.training_array[item],x_in,1,setup.no_of_inputs);
     copy_f_vector (tr_rec.norm_answers[item],desired_out,1,
	  setup.no_of_outputs);
     return;
     }

#endif