/*  Fille FFBRMPRC.C  Procedures for processing input to   */
/*                    broom balancing feedforward network. */

#ifndef FFBRMPRC_C
#define FFBRMPRC_C

#include <conio.h>
#include "ut.h"
#include "ff.h"
#include "trn.h"
#include "brm.h"

float x_factor,x_dot_factor,theta_factor,theta_dot_factor,force_factor;

short load_max_min_data (ff_setup_rec setup,state_rec *min_state,
	   state_rec *max_state,training_rec *tr_rec);
state_rec get_random_broom_state (state_rec max_state,state_rec min_state);
void get_network_input (ff_setup_rec setup,float force,state_rec state,
	  state_rec min_state,f_vector x_in);

/*  Implementation  */

short load_max_min_data (ff_setup_rec setup,state_rec *min_state,
	   state_rec *max_state,training_rec *tr_rec) {
      float delta;
      if (!process_training_file (setup.files.training_file_name,
	 setup.no_of_inputs,setup.no_of_outputs,tr_rec))
	 return 0;
      min_state->x_pos = x_pos_min;
      min_state->x_dot = tr_rec->min_in[2];
      min_state->theta = theta_low_lim;
      min_state->theta_dot = tr_rec->min_in[4];
      max_state->x_pos = x_pos_max;
      max_state->x_dot = tr_rec->max_in[2];
      max_state->theta = theta_up_lim;
      max_state->theta_dot = tr_rec->max_in[4];
      x_factor = 1.0/(max_state->x_pos - min_state->x_pos);
      delta = max_state->x_dot - min_state->x_dot;
      if (delta < 1e-6) {
	 cprintf ("\r\n\nProblem with x_dot data.  Press key to continue: ");
	 getch();
	 return 0;
	 }
      x_dot_factor = 1.0/delta;
      theta_factor = 1.0/(max_state->theta - min_state->theta);
      delta = max_state->theta_dot - min_state->theta_dot;
      if (delta < 1e-6) {
	 cprintf ("\r\n\nProblem with theta_dot data.  Press key to continue: ");
	 getch();
	 return 0;
	 }
      theta_dot_factor = 1.0/delta;
      force_factor = 1.0/(2 * force_mag);
      return 1;
      }  /* end proc */

state_rec get_random_broom_state (state_rec max_state,state_rec min_state) {
      state_rec the_state;
      the_state.x_pos = (max_state.x_pos - min_state.x_pos) * frand() +
	       min_state.x_pos;
      the_state.x_dot = (max_state.x_dot - min_state.x_dot) * frand() +
	       min_state.x_dot;
      the_state.theta = (max_state.theta - min_state.theta) * frand() +
	       min_state.theta;
      the_state.theta_dot = (max_state.theta_dot - min_state.theta_dot) *
	       frand() + min_state.theta_dot;
      return the_state;
      }  /* end funct */

void get_network_input (ff_setup_rec setup,float force,state_rec state,
	  state_rec min_state,f_vector x_in) {
     short i;
     float norm_in_max;
     x_in [1] = (state.x_pos - min_state.x_pos) * x_factor *
		setup.norm_in_range + setup.norm_in_min;
     x_in [2] = (state.x_dot - min_state.x_dot) * x_dot_factor *
		setup.norm_in_range + setup.norm_in_min;
     x_in [3] = (state.theta - min_state.theta) * theta_factor *
		setup.norm_in_range + setup.norm_in_min;
     x_in [4] = (state.theta_dot - min_state.theta_dot) * theta_dot_factor *
		setup.norm_in_range + setup.norm_in_min;
     x_in [5] = (force + force_mag) * force_factor * setup.norm_in_range +
		setup.norm_in_min;
     norm_in_max = setup.norm_in_min + setup.norm_in_range;
     for (i = 1;i<=5;i++) {
	 if (x_in [i] > norm_in_max) x_in [i] = norm_in_max;
	 if (x_in [i] < setup.norm_in_min) x_in [i] = setup.norm_in_min;
	 } /* end i */
     return;
     }  /* end proc */

#endif