/*  File FLGTRAIN.C  Train fuzzy system output rules using */
/*                   genetic optimization.  */

#ifndef FLGTRAIN_C
#define FLGTRAIN_C

#include <conio.h>
#include <alloc.h>

#include "ut.h"
#include "trn.h"
#include "flga.h"

void fl_genetic_training (tflg_setup_rec *fl,tga_setup_rec *ga_setup,
	tga_rec *ga);

/*  Implementation  */

void fl_genetic_training (tflg_setup_rec *fl,tga_setup_rec *ga_setup,
	tga_rec *ga) {
   int gen = 0;
   const int one_fl_output = 1;
   short cancel_flag = 0,new_optimum = 0,within_tolerance = 0;
   int i;
   clrscr();
   textcolor (YELLOW);
   gotoxy (1,5);
     if (fl->fuzzy_system_file_name[0] == '\0') {
	cprintf ("\r\nFuzzy System File Name is blank.\r\n");
	cputs ("Press key to cancel: ");
	getch();
	return;
	}
     if (fl->training_file_name[0] == '\0') {
	cprintf ("\r\nTraining File Name is blank.\r\n");
	cputs ("Press key to cancel: ");
	getch();
	return;
	}
   cprintf ("Fuzzy - Genetic Modeling");
   cprintf ("\r\n\nNumber of Inputs: %d",fl->no_of_fl_inputs);
   cprintf ("\r\n\nNumber of Input Fuzzy Sets: %d",fl->no_of_fl_inp_regions);
   cprintf ("\r\n\nPopulation Size: %d",ga_setup->population_size);
   cprintf ("\r\n\nBit Length: %d       Chromosome Length: %d",
       ga_setup->bit_len,ga_setup->chrom_len);
   cprintf ("\r\n\nFuzzy System File Name: %s",fl->fuzzy_system_file_name);
   cprintf ("\r\n\nTraining File Name: %s",fl->training_file_name);
   cprintf ("\r\n\nPress key to continue (esc to cancel): ");
   if (getch() == ESC_KEY) return;
   init_flga_fuzzy_system (*fl,&g_fuzzy_system);
   /* fl_evaluation_fn needs to use global g_tr_rec:  */
   if (!process_training_file (fl->training_file_name,
       fl->no_of_fl_inputs,one_fl_output,&g_tr_rec)) goto exit_proc;
   g_tr_rec.norm_error_scaling =
     1.0/(g_tr_rec.no_of_training_items * (fl->norm_out_range));
   clrscr();
   gotoxy (1,5);
   cprintf ("Initializing system...");
   normalize_training_set (fl->no_of_fl_inputs,one_fl_output,
      fl->norm_in_range,fl->norm_in_min,fl->norm_out_range,fl->norm_out_min,
      g_tr_rec);
   if (!initialize_genetic_alg (ga_setup,ga,&fl_evaluation_fn))
      goto exit_proc;
   cprintf ("\r\n\nStarting first generation...");
   do {
      gen += 1;
      generation (ga_setup,ga,&fl_evaluation_fn);
      statistics (*ga_setup,ga->populations[ga->g_new],ga,&new_optimum);
      if (new_optimum) {
	 ga_vector_to_fuzzy_rules (ga->opt_vector,&g_fuzzy_system);
	 write_fuzzy_system (fl->fuzzy_system_file_name,g_fuzzy_system);
	 }
      display (gen,*ga_setup,*ga,&cancel_flag,&within_tolerance);
      ga->g_old = (ga->g_old + 1)%2;
      ga->g_new = (ga->g_new + 1)%2;
   } while ((gen <= ga_setup->max_gens) && (!within_tolerance)
	    && (!cancel_flag));
exit_proc:
   free_fuzzy_rules (&g_fuzzy_system);
   free_ga_pointers (*ga_setup,ga);
   free_training_pointers(&g_tr_rec);
   cprintf ("\r\nFLGTRAIN:  heapcheck = %d",heapcheck());
   getch();
   return;
   }  /* end proc */

#endif